/*
 * file: ion_Kh_eulexp.c
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include <stdlib.h>
#include <neurocad/module.h>

enum __chan_parvar{
	_par_mconduct = 0,
	_par_rev_pot,

	_par_h_v12,
	_par_h_k,
	_par_h_tau1,
	_par_h_tau2,
	_par_h_contrib1,
	_par_h_contrib2,

	_par_temperature,

	_var_voltage,
	_var_current,
	_var_gate_h1,
	_var_gate_h2,
	_var_gate_h,
	_var_conduct,
	_var_total_vcurrent,
	_var_total_conduct,
	_var_total_GE,
#ifdef __DBFLAG__
	_db_var_steadystate_h,
	_db_var_tau_h1,
	_db_var_tau_h2,
#endif
	__end_par_var__,
};

typedef struct __block{
	__MODULE
	dptr parvar[__end_par_var__];
	struct __block *nextblk;
} block;

const enum __mt modtype = __channel;
const int moditer = 1;
const int modprior = 2;
const int modnames = __end_par_var__;
const int	modblocksize	= sizeof(block);
const double	modweight	= -1.0;

const char modname[] = "Generic Kh current. Spain, W.J., Schwindt, P.C., Crill, W.E.\0";

const char moddescription[]= "The module description.\nThis module contains the implementation anomalous rectifier potassium ion channel.\nTime constants were measured at 36 degC\nThe channel description was taken from \nAnomalous rectification in neurons from cat sensorimotor cortex in vitro.\nSpain, W.J., Schwindt, P.C., Crill, W.E.\nJ. Neurophysiol. 57(5): 1555-1576, 1987.\nSee also:\nAmplification and linearization of distal synaptic input to cortical pyramidal cells.\nBernander, O., Koch, C., Douglas, R.J.\nJ. Neurophysiol. 72(6): 2743-2753, 1994.\nModels of subthreshold membrane resonance in neocortical neurons.\nHutcheon, B., Miura, R.M., Puil, E.\nJ. Neurophysiol. 76(2): 698-714, 1996.\0";

const char modpar[] = "Maximal conductance\tReversal potential\tInactivation half-voltage\tInactivation K\tTime constant for voltage-dependent inactivation gate 1\tTime constant for voltage-dependent inactivation gate 2\tContribution to voltage-dependent inactivation gate 1\tContribution to voltage-dependent inactivation gate 2\tTemperature coefficient\0";
#ifdef __DBFLAG__
const char modvar[] = "Membrane potential\tChannel current\tVoltage-dependent inactivation gate 1\tVoltage-dependent inactivation gate 2\tVoltage-dependent inactivation gate\tChannel conductance\tVoltage-dependent cell current\tCell conductance\tTotal product of channel conductances and channel reversal potentials\t__DB__ inactivation steady-state\t__DB__ inactivation time constant 1\t__DB__ inactivation time constant 2\0";
#else
const char modvar[] = "Membrane potential\tChannel current\tVoltage-dependent inactivation gate 1\tVoltage-dependent inactivation gate 2\tVoltage-dependent inactivation gate\tChannel conductance\tVoltage-dependent cell current\tCell conductance\tTotal product of channel conductances and channel reversal potentials\0";
#endif


static int mid = -1;
static block *rootblk = NULL;
static int iteration = 0;

int init( int imid )
{
	mid = imid;
	rootblk = NULL;
	iteration = 0;
	return 0;
}

int release ( int imid )
{
	rootblk = NULL;
	return 0;
}

module *add( int imid, module *next, void *mem_handle )
{
	block * newblk = ( block *)mem_g_malloc( mem_handle, sizeof( block ) );
	if( newblk == NULL )
		return ( module *) NULL;
	newblk->nextblk = rootblk;
	rootblk = newblk;
	int cnt; for( cnt = 0; cnt < __end_par_var__; rootblk->parvar[cnt++].dp = NULL );
	rootblk->mid = mid;
	rootblk->next = next;
	return ( module *)rootblk;
}

int calculate( int imid )
{
	if(rootblk == NULL) return -1;
	block *currentblk = rootblk;
	double 	voltage, revpotential, newconduct, newcur,
		th1, th2, h, newgateh1, newgateh2, newgateh;
	for( ; currentblk != NULL; currentblk = currentblk->nextblk ){
		voltage = GN_SW( _var_voltage );
		revpotential = GN_SW( _par_rev_pot );
		newconduct = GN_SW( _par_mconduct );

		newcur = voltage-revpotential;
		th1 = GN_SW( _par_h_tau1 )*GN_SW( _par_temperature );
		th2 = GN_SW( _par_h_tau2 )*GN_SW( _par_temperature );
		h = 1.0/( 1.0 + exp(-( voltage-GN_SW( _par_h_v12 ))/GN_SW( _par_h_k )));
		GNASW( _var_gate_h1 ) = 
		newgateh1 = ( GN_SW( _var_gate_h1 )-h )*exp( -time_step/th1 )+h;
		GNASW( _var_gate_h2 ) = 
		newgateh2 = ( GN_SW( _var_gate_h2 )-h )*exp( -time_step/th2 )+h;
		newgateh = GN_SW( _par_h_contrib1 )*newgateh1+
				GN_SW( _par_h_contrib2 )*newgateh2;
		GNASW( _var_gate_h ) = newgateh;
		newconduct *= newgateh;
		newcur *= newconduct;

		GNASW( _var_conduct ) = newconduct;
		GNASW( _var_total_conduct ) += newconduct;
		GNASW( _var_total_GE ) += newconduct*revpotential;
		GNASW( _var_current ) = newcur;
		GNASW( _var_total_vcurrent ) += newcur;
#ifdef __DBFLAG__
		GNASW( _db_var_steadystate_h )	= h;
		GNASW( _db_var_tau_h1 )		= th1;
		GNASW( _db_var_tau_h2 )		= th2;
#endif
	}
	return iteration;
}

int slack( int imid )
{
	if( rootblk != NULL )
		return iteration;
	return -1;
}

int restore( int imid, int hFile, int oid, int bid )
{
	return 0;
}

int save( int imid, int hFile, int oid, int bid )
{
	return 0;
}
const enum __ml modmulti        = __single;
