/*
 * file: ion_NaP_eulexp.c
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include <stdlib.h>
#include <neurocad/module.h>

enum __chan_parvar{
	_par_mconduct = 0,
	_par_rev_pot,
	
	_par_m_A,
	_par_m_B,
	_par_m_C,
	_par_m_D,
	_par_m_E,
	_par_m_F,
	_par_m_G,
	_par_m_H,
	_par_temperature,
	
	_var_voltage,
	_var_current,
	_var_gate_m,
	_var_conduct,
	_var_total_vcurrent,
	_var_total_conduct,
	_var_total_GE,
#ifdef __DBFLAG__
	_db_var_alpha_m,
	_db_var_beta_m,
	_db_var_steadystate_m,
	_db_var_tau_m,
#endif
	__end_par_var__,
};

typedef struct __block{
	__MODULE
	dptr parvar[__end_par_var__];
	struct __block *nextblk;
} block;

const enum __mt modtype = __channel;
const int moditer = 1;
const int modprior = 2;
const int modnames = __end_par_var__;
const int	modblocksize	= sizeof(block);
const double	modweight	= -1.0;

const char modname[] = "Generic NaP current. De Schutter and Bower.\n";

const char moddescription[]= "The module description.\nThis module contains the implementation of persistent sodium ion channel.\nThe channel description was taken from paper\nAn Active Membrane Model of the Cerebellar Purkinje Cell I. Simulation of Current Clamps in Slice\nEric De Schutter and James M. Bower\nJ. Neurophysiol. Vol. 71, No. 1, 1994.\0";

const char modpar[] = "Maximal conductance\tReversal potential\tActivation A\tActivation B\tActivation C\tActivation D\tActivation E\tActivation F\tActivation G\tActivation H\tTemperature coefficient\0";

#ifdef __DBFLAG__
const char modvar[] = "Membrane potential\tChannel current\tVoltage-dependent activation gate\tChannel conductance\tVoltage-dependent cell current\tCell conductance\tTotal product of channel conductances and channel reversal potentials\t__DB__ activation alpha\t__DB__ activation beta\t__DB__ activation steady-state\t__DB__ activation time constant\0";
#else
const char modvar[] = "Membrane potential\tChannel current\tVoltage-dependent activation gate\tChannel conductance\tVoltage-dependent cell current\tCell conductance\tTotal product of channel conductances and channel reversal potentials\0";
#endif


static int mid = -1;
static block *rootblk = NULL;
static int iteration = 0;

int init( int imid )
{
	mid = imid;
	rootblk = NULL;
	iteration = 0;
	return 0;
}

int release ( int imid )
{
	rootblk = NULL;
	return 0;
}

module *add( int imid, module *next, void *mem_handle )
{
	block * newblk = ( block *)mem_g_malloc( mem_handle, sizeof( block ) );
	if( newblk == NULL )
		return ( module *)NULL;
	newblk->nextblk = rootblk;
	rootblk = newblk;
	int cnt; for( cnt = 0; cnt < __end_par_var__; rootblk->parvar[cnt++].dp = NULL );
	rootblk->mid = mid;
	rootblk->next = next;
	return ( module *)rootblk;
}

int calculate( int imid )
{
	if( rootblk == NULL ) return -1;
	block *currentblk = rootblk;
	double 	voltage, revpotential, newconduct, newcur,
		alpham, betam, tm, m, newgatem;
	for( ; currentblk != NULL; currentblk = currentblk->nextblk ){
		voltage = GN_SW( _var_voltage );
		revpotential = GN_SW( _par_rev_pot );
		newconduct = GN_SW( _par_mconduct );

		newcur = voltage-revpotential;
		alpham = GN_SW( _par_m_A )/
			( GN_SW( _par_m_B )+exp(( voltage+GN_SW(_par_m_C))/GN_SW( _par_m_D )));
		betam  = GN_SW( _par_m_E )/
			( GN_SW( _par_m_F )+exp(( voltage+GN_SW(_par_m_G))/GN_SW( _par_m_H )));
		tm = 1./( alpham+betam);
		m = alpham*tm;				/*alpha/(alpha+beta)*/
		tm *= GN_SW( _par_temperature );
		newgatem = ( GN_SW( _var_gate_m )-m )*exp( -time_step /tm )+m;
		GNASW( _var_gate_m ) = newgatem;
		newconduct *= ( newgatem*newgatem*newgatem );
		newcur *= newconduct;
		
		GNASW( _var_conduct ) = newconduct;
		GNASW( _var_total_conduct ) += newconduct;
		GNASW( _var_total_GE ) += newconduct*revpotential;
		GNASW( _var_current ) = newcur;
		GNASW( _var_total_vcurrent ) += newcur;
#ifdef __DBFLAG__
		GNASW( _db_var_alpha_m )	= alpham;
		GNASW( _db_var_beta_m )		= betam;
		GNASW( _db_var_steadystate_m )	= m;
		GNASW( _db_var_tau_m )		= tm;
#endif
	}
	return iteration;
}

int slack( int imid )
{
	if( rootblk != NULL )
		return iteration;
	return -1;
}

int restore( int imid, int hFile, int oid, int bid )
{
	return 0;
}

int save( int imid, int hFile, int oid, int bid )
{
	return 0;
}
const enum __ml modmulti        = __single;
