/*
 * file: conc_eulexp.c
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <neurocad/module.h>

enum {
	_par_conc_out = 0,
	_par_conc_in_inf,
	_par_alpha,
	_par_msquare,
	_par_tau,
	
	_var_potential,
	_var_current,
	_var_conc_in,
#ifdef __DBFLAG__
	_db_var_steadystate_ca,
#endif
	__end_par_var__
};

typedef struct __block{
	__MODULE
	dptr	parvar[__end_par_var__];
	struct __block *nextblk;
} block;

const enum __mt	modtype	= __cell2h;
const enum __ml modmulti = __single;
const int	moditer		= 1;
const int	modprior	= 1;
const int	modnames	= __end_par_var__;
const int	modblocksize	= sizeof(block);
const double	modweight	= -1.0;

const char	modname[] 	= "Fist-order Ca-concentration kinetics.\0";
const char	moddescription[]= "This module contains the implementation of the fist-order kinetics of calcium intracellular concentration.\nThe kinetic description was taken from paper\nA. Compte et al. J.Neurophysiol 89: 2707-2725, 2003.\nThe different equasion are solved by exponentioal Euler method and compabilite with compartment/single sell module.\0";

const char	modpar[] 	= 
		"Outside calcium concentration"
		"\tInside infinity calcium concentration"
		"\tCoefficient alpha"
		"\tMembrane square"
		"\tPump time constant\0";

const char	modvar[] 	= 
		"Ca potential"
		"\tCalcium current"
		"\tInside calcium concentration"
#ifdef __DBFLAG__
		"\t__DB__ Ca inside steady-state"
#endif
		"\0";

static int mid;	/*module id*/
static block *rootblk;


int init( int iMID )
{
	mid = iMID;
	rootblk  = NULL;
	return 0;
}

int release ( int iModID )
{
	//The module uses the static memory function mem_g_malloc()
	//Thus all resources must be free by memmanager.c;
	rootblk =NULL;
	return 0;
}

module *add( int iMID, module *pMod, void *memory_handle)
{
	block * newblk = (block *) mem_g_malloc(memory_handle, sizeof( block ));
	if(newblk == NULL) return (module *) NULL; /*Cannot malloc memory*/
	newblk->nextblk = rootblk;
	rootblk = newblk;
	int cnt; for(cnt=0; cnt<__end_par_var__; cnt++)rootblk->parvar[cnt].dp=NULL;
	rootblk->mid = mid;
	rootblk->next = pMod;
	return (module *) rootblk;
}


int calculate( int iMID )
{
	if(rootblk == NULL) return -1;
	block *currentblk;
	datum inInf, tau, incon, influx;
	for( currentblk = rootblk; currentblk != NULL; currentblk = currentblk->nextblk){
		tau = GN_SW(_par_tau);
		influx =  - GN_SW(_par_alpha) * GN_SW(_var_current) * GN_SW(_par_msquare); 
		if( influx < 0.0) influx = 0.0;
/*		inInf = GN_SW(_par_conc_in_inf) + influx/tau;
		incon = GNASW(_var_conc_in) = inInf + 
			( GN_SW( _var_conc_in )-inInf )*exp( -time_step/tau );
*/
/*		incon = GNASW(_var_conc_in) = 
			GN_SW( _var_conc_in )*( 1.0 - time_step/tau ) + ( influx + GN_SW(_par_conc_in_inf)/tau)*time_step;
*/
		datum c = GN_SW(_var_conc_in);
		incon = GNASW(_var_conc_in) = c + (influx + (GN_SW(_par_conc_in_inf) - c)/tau)*time_step;
//		GNASW(_var_potential) = 13.27 * log( GNASW( _par_conc_out ) / incon );
		GNASW(_var_current) = 0.0;
#ifdef __DBFLAG__
		GNASW( _db_var_steadystate_ca ) = inInf;
#endif
	}
	return 0;
}

int slack( int iMID )
{
	if(rootblk == NULL) return -1;
	return 0;
}

int restore( int iMID, int hFile, int oid, int bid )
{
	return 0;
}

int save( int iMID, int hFile, int oid, int bid )
{
	return 0;
}
