/*
 * file: ion_channel.tpl
 * 
 * Authors: Sergey Markin <markin@nisms.krinc.ru>
 * 
 * ----------------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ----------------------------------------------------------------------------
 *
 * 				General information.
 * 				--------------------
 *
 * This file contains a set of basic functions that implements interface between
 * a model ionic channel and the simulation engine, namely:
 * 
 * int init( int iMID );
 * int release( void );
 * module *add( module *pMod, void *memory_header );
 * int del( int iOID, int iBID );
 * int calculate(int iMID );
 * int slack( int iMID );
 * int restore( int iMID, int hFile, int oid, int bid );
 * int save( int iMID, int hFile, int oid, int bid );
 * These functions may be hidden if you define correspondent preprocessor macroses,
 * which are:
 * 
 * __MOD_INIT
 * __MOD_RELEASE
 * __MOD_ADD
 * __MOD_DEL
 * __MOD_CALCULATE
 * __MOD_SLACK
 * __MOD_RESTORE
 * __MOD_SAVE
 *
 * It means if you wish to define your own function instead a basic one you 
 * have to specify a correspondent preprocessor macros (for example: 
 * #define __MOD_INIT and/or #define __MOD_RELEASE etc). In this case the 
 * postfix '_base' will be added to the name of the correspondent function and
 * you may use the basic function (named x_base) in your own code only. In fact,
 * functions "init(), release(), del(), add(), and slack()" are pretty standard 
 * and should not be redefined in the most cases, contrary to function 
 * "calculate()" that must be specified for each module. Here you can find the
 * implementation of "calculate()" function .......
 * 
 * 				Gate variables.
 *				---------------
 * dGATE(v)
 * --------- = ......							(1)
 *    dt
 * 
 * GATEinf(v)	
 * Tau(v)		
 * Exponential Euler...
 * 
 * 				Channel current.
 *				---------------
 * The channel current (Ichan) is calculated according follow equation
 * 
 * Ichan = Gmax*( v-u)*GATEa(v)^POWa*GATEi(v)^POWi			(2)
 * where,
 * Gmax 	- maximal conductance
 * v 		- membrane potential
 * u 		- reversal potential
 * GATEa(v)	- activation
 * POWa
 * GATEi(v)	- inactivation
 * POWi
 * 
 * 				Specific gate variables:
 * 				-----------------------
 * 1. Specific alpha-beta activation/inactivation.
 * You may specify __ALPHA_M(v) and __BETA_M(v)/__ALPHA_H(v) and __BETA_H(v)
 * macroses* if you want to use specific alpha-beta gate variable for 
 * activation and/or inactivation. The correspondent gate variable will be 
 * calculate according to equation (1), where
 * 
 * GATEinf 	= alpha(v)/(alpha(v)+beta(v))
 * Tau**	= 1/(alpha(v)+beta(v))
 * ______
 * Note: * You must specify __ALPHA_x(v) and __BETA_x(v) macroses together
 * otherwise the correspondent gate variable will not be processed.
 * ** Time constant Tau(v) will not be processed if you specify macros
 * __NON_TAU_M (see section Other macroses)
 * 
 * 2. Specific activation/inactivation
 * You must specify macoses __INF_M(v)/__INF_H(v) for correspondent gate 
 * variable (activation/inactivation) and macroses __TAU_M(v)/
 * __TAU_H(v)* for correspondent time constants (Tau). If you do not 
 * specify __TAU_M(v)/__TAU_H(v) macroses the equation (1) will not be
 * processed and correspondent gate variable will be equal GATEinf.
 * ____
 * Note: *Even if you specify __TAU_M(v)/__TAU_H(v) macroses time constant 
 * Tau(v) will not be processed if you specify macros __NON_TAU_M 
 * (see section Other macroses)
 * 
 * 				Generic gate variables:
 * 				----------------------
 * 1. Generic alpha-beta activation/inactivation.
 * You must specify macroses __GEN_ALPHA_BETA_M/__GEN_ALPHA_BETA_H if you want
 * to use generic alpha-beta gate variable for activation and/or inactivation.
 * In this case the parameters [see paper????] for the correspondent generic 
 * alpha-beta gate variable will be added to the parameters list of the ionic 
 * channel. The correspondent gate variable will be processed according to the 
 * paper ????. 
 * For example: if you specify __GEN_ALPHA_BETA_M macros the parameters
 * will be added to the parameters list of the ionic channel (see tab. 1)
 *   Table 1.
 *   --------------------------------------------------------------
 * 	      index		|		name
 *   --------------------------------------------------------------
 *	_par_alpha_m_A,		|
 *	_par_alpha_m_B,		|
 *	_par_alpha_m_C,		|
 *	_par_alpha_m_D,		|
 *	_par_alpha_m_E,		|
 *	_par_beta_m_A,		|
 *	_par_beta_m_B,		|
 *	_par_beta_m_C,		|
 *	_par_beta_m_D,		|
 *	_par_beta_m_E,		|
 *   --------------------------------------------------------------
 * and the correspondent gate variable will be calculate according to 
 * equation (1), where
 * 
 * GATEinf(v)	= alpha(v)/(alpha(v)+beta(v)),
 * Tau(v)*	= 1/(alpha(v)+beta(v)), where
 * 	v is membrane potential,
 * 	alpha(v) = (x_A+x_B*v)/(x_C+exp((v+x_D)/x_E)),
 * 	beta(v)  = (x_A+x_B*v)/(x_C+exp((v+x_D)/x_E)).
 * ______
 * Note: *Time constant Tau(v) will not be processed if you specify macros
 * __NON_TAU_M (see section Other macroses)
 * 
 * 2. Generic activation/inactivation.
 * You must specify macroses __GENERIC_M/__GENERIC_H if you want to 
 * use generic gate variable for activation and/or inactivation.
 * In this case the parameters [see paper????] for the generic gate 
 * variable will be added to the parameters list of the ionic channel. 
 * The correspondent gate variable will be processed according the paper
 * ????:
 * For example: if you specify __GENERIC_M macros the parameters
 * will be added to the parameters list of the ionic channel (see tab. 2)
 *   Table. 2
 *   --------------------------------------------------------------
 * 	      index		|		name
 *   --------------------------------------------------------------
 *	_par_gen_m_v12		|	    half-voltage
 *	_par_gen_m_k		|
 *	_par_gen_m_Tmax*	|
 *	_par_gen_m_Tmax1*	|
 *	_par_gen_m_Tmax2*	|
 *   --------------------------------------------------------------
 * and the correspondent gate variable will be calculate according to 
 * equation (1), where:
 * 
 * GATEinf(v)	= 1./( 1.+exp( -( v-x_v12 )/( x_k )))
 * Tau(v)*	= 2*x_Tmax/(exp(-(v-x_Tmax1)/x_Tmax1)+exp(-(v-x_Tmax2)/x_Tmax2))
 * ______
 * Note: *Parameters will be excluded from parameters list and time constant 
 * Tau(v) will not be processed if you specify macros __NON_TAU_M (see section 
 * Other macroses)
 *
 * 	The priority of the macroses described above is:
 * 	------------------------------------------------
 * __ALPHA_x(v) and __BETA_x(v)			max
 * __INF_x(v)					 |
 * __GEN_ALPHA_BETA_x				 v
 * __GENERIC_x					min
 * 
 * 				Other macroses:
 * 				---------------
 * Macroses __TEMPERATURE_M/__TEMPERATURE_H set up additional parameters 
 * m temperature coefficient/h temperature coefficient which are temperature
 * coefficient for correspondent gate variables. Note, if you specify macroses 
 * __NON_TAU_x the parameters will be excluded from parameters list.
 * 
 * Sometimes the equation (1) should not be processed and gate variable assume
 * to be equal the correspondent GATEinf. You may define __NON_TAU_M/__NON_TAU_H 
 * macroses which help you to skip the equation (1). 
 * 
 * You can specify macroses __CONST_TAU_x if you want to fixed Tau(v) in some
 * constant for activation and/or inactivation. In this case macroeses __TEMPERATURE_x
 * and __TAU_x will disable. If you set macroses __NON_TAU_x, __CONST_TAU_x will be
 * desable.
 * 
 * In order to process equation (2) you have to specify POWa and POWi variables.
 * You may define you own macroses to calculate GATEx(v)^POWx. These are 
 * __POW_M(x)/__POW_H(x) macroses. You also may define __GENERIC_POW_M/
 * __GENERIC_POW_H macroses. In this case the parameters _par_m_pow/_par_h_pow
 * will be added to the parameters list of the ionic channel. If you do not 
 * specify these macroses (either __POW_M(x)/__POW_H(x) or __GENERIC_POW_M/
 * __GENERIC_POW_H) POWx will be equal 1. The priority of the macroses described
 * above is:
 * __POW_x -> __GENERIC_POW_x	
 *   max   ->   min
 *
 * You can specify __ACCOMULATE for accomulation currents of same sort ion is 
 * named varibale. Corresponedent name 'accomulate ion current' will be made
 * for order of accomulation
 * 
 * Macroses __MOD_NAME and __MOD_DESCRIPTION define the modname and 
 * moddescription variables which describe the general properties of a module.
 * 
 * 				Aknowlegment:
 * 				-------------
 * This code is based on files ion_K_slow_eulexp.c and ion_Na_fast_eulexp.c
 * invented by Ruben Tikidji-Hamburian <rth@nisms.krinc.ru>. You should see
 * these files if you want to understand the basic principles how to develop
 * you own module of an ionic channel.
 * 
 * Good luck
 * ----------------------------------------------------------------------------
 * VERY IMPORTANT: In order to produce the C-code to be compile you MUST first 
 * compile the preliminary file with flags -E -P. The result of "pre" compilation 
 * may be used by usual way.
 * 
 * For example:
 * you have file pre_example.c
 * -----------------------------------------------------------------------------------
 * #define __ALPHA_M(__U) (0.01 *((__U) + 34.0)/(1.0 - exp(-((__U)+34.0)/10.0) ))
 * #define __BETA_M(__U)  (0.125 * exp(- ((__U)  + 44.0)/25.0))
 *
 * #define __MOD_NAME "Example of K-slow channel for cell_2h model"
 * #define __MOD_DESCRIPTION "Description of module.\nThis module contains Slow K channel for the Hodgkin-Huxley neuron(or compartment)[See for example cell_h2.so]. The main equation for \"K-slow gate\"(n) is solved by exponential Euler method.\0"
 *
 * #include "ion_channel.tpl"
 *
 * -----------------------------------------------------------------------------------
 *
 * type gcc -E -P -o example.c pre_example.c
 * 
 * The result of "pre" compilation (file example.c) contains the "pure" C-code which
 * is possible to use in your project.
 */

#define __XSTRING(__S...)	__STRING(__S)
#define __STRING(__S...)	#__S
#define __XCNCT(__S1,__S2)	__CNCT(__S1,__S2)
#define __CNCT(__S1,__S2)	__S1##__S2
#define __APPEND_NAME(__S0,__S)	__XCNCT(__ADD_TAB(__S0),__S)
#define __ADD_ENTER(__S)	__S\n
#define __ADD_NULL(__S)		__S\0
#define __ADD_TAB(__S)		__S\t
#define __XINCLUDE(__S,__X)	__SYMBOL(__S)__INCLUDE(__X)
#define __INCLUDE(__X)		include __X
#define __SYMBOL(__S...)	__S

// __GATE_M
#if defined( __ALPHA_M ) && defined( __BETA_M )
  #undef __INF_M
  #undef __GEN_ALPHA_BETA_M
  #undef __GENERIC_M
  #undef __TAU_M
  #define __GATE_M
  #define __TAU_M(__U)		/*will be calculate as 1/(alpha+beta)*/
#elif defined( __INF_M )
  #undef __ALPHA_M
  #undef __BETA_M
  #undef __GEN_ALPHA_BETA_M
  #undef __GENERIC_M
  #define __GATE_M
#elif defined(__GEN_ALPHA_BETA_M )	/*generic alpha/beta activation gate*/
  #undef __ALPHA_M
  #undef __BETA_M
  #undef __INF_M
  #undef __GENERIC_M
  #undef __TAU_M
  #define __GATE_M
  #define __ALPHA_M(__U)	(GN_SW(_par_alpha_m_A)+GN_SW(_par_alpha_m_B)*(__U))/ \
				(GN_SW(_par_alpha_m_C)+exp(((__U)+GN_SW(_par_alpha_m_D))/\
				 GN_SW(_par_alpha_m_E)))
  #define __BETA_M(__U)		(GN_SW(_par_beta_m_A)+GN_SW(_par_beta_m_B)*(__U))/ \
				(GN_SW(_par_beta_m_C)+exp(((__U)+GN_SW(_par_beta_m_D))/\
				 GN_SW(_par_beta_m_E)))
  #define __TAU_M(__U)	/*will be calculate as 1/(alpha+beta)*/
#elif defined( __GENERIC_M )		/*generic activation*/
  #undef __ALPHA_M
  #undef __BETA_M
  #undef __INF_M
  #undef __GEN_ALPHA_BETA_M
  #undef __TAU_M
  #define __GATE_M
  #define __INF_M(__U)		1.0/(1.0+exp(-((__U)-GN_SW(_par_gen_m_v12))/\
				GN_SW(_par_gen_m_k)))
  #define __TAU_M(__U)		2*GN_SW(_par_gen_m_Tmax)/\
				(exp(-((__U)-GN_SW(_par_gen_m_Tmax1))/\
				GN_SW(_par_gen_m_Tmax1))+exp(-((__U)-GN_SW( _par_gen_m_Tmax2))/\
				GN_SW(_par_gen_m_Tmax2)))
#else 
  #undef __GATE_M
#endif /* defined( __ALPHA_M ) && defined( __BETA_M ) */
#ifdef __GATE_M 
  #ifndef __POW_M
     #ifdef __GENERIC_POW_M
        #define __POW_M(__X)	pow(__X, GN_SW( _par_h_pow ))
     #else
        #define __POW_M(__X)	(__X)
     #endif /* __GENERIC_POW_M */
  #endif /* __POW_M */
  #ifdef __NON_TAU_M
     #undef __CONST_TAU_M
     #undef __TAU_M
     #undef __TEMPERATURE_M
  #endif /* __NON_TAU_M */
  #ifdef __CONST_TAU_M
    #define __TAU_M
    #undef __TEMPERATURE_M
  #endif /* __CONST_TAU_M */
#endif /* __GATE_M */
// __GATE_M 

// __GATE_H
#if defined( __ALPHA_H ) && defined( __BETA_H )
  #undef __INF_H
  #undef __GEN_ALPHA_BETA_H
  #undef __GENERIC_H
  #undef __TAU_H
  #define __GATE_H
  #define __TAU_H(__U)		/*will be calculate as 1/(alpha+beta)*/
#elif defined( __INF_H )
  #undef __ALPHA_H
  #undef __BETA_H
  #undef __GEN_ALPHA_BETA_H
  #undef __GENERIC_H
  #define __GATE_H
#elif defined( __GEN_ALPHA_BETA_H )	/*generic alpha/beta inactivation gate*/
  #undef __ALPHA_H
  #undef __BETA_H
  #undef __INF_H
  #undef __GENERIC_H
  #undef __TAU_H
  #define __GATE_H
  #define __ALPHA_H(__U)	(GN_SW(_par_alpha_h_A)+GN_SW(_par_alpha_h_B)*(__U))/ \
				(GN_SW(_par_alpha_h_C)+exp(((__U)+GN_SW(_par_alpha_h_D))/\
				 GN_SW(_par_alpha_h_E)))
  #define __BETA_H(__U)		(GN_SW(_par_beta_h_A)+GN_SW(_par_beta_h_B)*(__U))/ \
				(GN_SW(_par_beta_h_C)+exp(((__U)+GN_SW(_par_beta_h_D))/\
				 GN_SW(_par_beta_h_E)))
  #define __TAU_H(__U)		/*will be calculate as 1/(alpha+beta)*/
#elif defined( __GENERIC_H )		/*generic inactivation*/
  #undef __ALPHA_H
  #undef __BETA_H
  #undef __INF_H
  #undef __GEN_ALPHA_BETA_H
  #undef __TAU_H
  #define __GATE_H
  #define __INF_H(__U)		1./(1.+exp(-((__U)-GN_SW(_par_gen_h_v12))/\
				GN_SW(_par_gen_h_k)))
  #define __TAU_H(__U)		2*GN_SW(_par_gen_h_Tmax)/\
				(exp(-((__U)-GN_SW(_par_gen_h_Tmax1))/\
				GN_SW(_par_gen_h_Tmax1))+exp(-(__U)-GN_SW(_par_gen_h_Tmax2))/\
				GN_SW(_par_gen_h_Tmax2)))
#else 
  #undef __GATE_H
#endif /*defined( __ALPHA_H ) && defined( __BETA_H )*/
#ifdef __GATE_H
  #ifndef __POW_H
     #ifdef __GENERIC_POW_H
        #define __POW_H(__X)	pow(__X, GN_SW( _par_h_pow ))
     #else
        #define __POW_H(__X)	(__X)
     #endif /* __GENERIC_POW_H */
  #endif /* __POW_H */
  #ifdef __NON_TAU_H
     #undef __CONST_TAU_H
     #undef __TAU_H
     #undef __TEMPERATURE_H
  #endif /* __NON_TAU_H */
  #ifdef __CONST_TAU_H
     #define __TAU_H
     #undef __TEMPERATURE_H
  #endif /* __CONST_TAU_H */
#endif /* __GATE_H */
// __GATE_H

__XINCLUDE(#,<stdlib.h>)
__XINCLUDE(#,<neurocad/module.h>)

enum __chan_parvar{
// setup parameters and modpar string
#define __BASIC_PARS	Maximal conductance\tReversal potential
	_par_mconduct	= 0,		// maximal conductance
	_par_rev_pot,			// reversal potential
#ifdef __GATE_M
  #ifdef __GEN_ALPHA_BETA_M /* parameters of generic alpha/beta activation*/
     #define __ALT_M_PARS	__APPEND_NAME(__BASIC_PARS, _par_alpha_m_A\t_par_alpha_m_B\t_par_alpha_m_C\t_par_alpha_m_D\t_par_alpha_m_E\t_par_beta_m_A\t_par_beta_m_B\t_par_beta_m_C\t_par_beta_m_D\_par_beta_m_E)
	_par_alpha_m_A,			//
	_par_alpha_m_B,			//
	_par_alpha_m_C,			//
	_par_alpha_m_D,			//
	_par_alpha_m_E,			//
	_par_beta_m_A,			//
	_par_beta_m_B,			//
	_par_beta_m_C,			//
	_par_beta_m_D,			//
	_par_beta_m_E,			//
  #elif defined( __GENERIC_M ) /* parameters of generic activation*/
	_par_gen_m_v12,			// half-voltage
	_par_gen_m_k,			//
     #ifdef __TAU_M
        #define __ALT_M_PARS	__APPEND_NAME(__BASIC_PARS, _par_gen_m_v12\t_par_gen_m_k\t_par_gen_m_Tmax\t_par_gen_m_Tmax1\t_par_gen_m_Tmax2)
	_par_gen_m_Tmax,		//
	_par_gen_m_Tmax1,		//
	_par_gen_m_Tmax2,		//
     #else
        #define __ALT_M_PARS	__APPEND_NAME(__BASIC_PARS, _par_gen_m_v12\t_par_gen_m_k)
     #endif /* __TAU_M */
  #else
     #define __ALT_M_PARS	__BASIC_PARS
  #endif /* __GEN_ALPHA_BETA_M */
  #ifdef __GENERIC_POW_M
     #define __POW_M_PAR	__APPEND_NAME(__ALT_M_PARS,_par_m_pow)
	_par_m_pow,			//
  #else
     #define __POW_M_PAR	__ALT_M_PARS
  #endif /* __GENERIC_POW_M */
  #if defined( __TEMPERATURE_M ) && defined ( __TAU_M )
     #define __T_M_PAR	__APPEND_NAME(__POW_M_PAR,m temperature coefficient)
	_par_m_temperature,		// temperature coefficient
  #else
     #define __T_M_PAR	__POW_M_PAR
  #endif /* defined( __TEMPERATURE_M ) && defined ( __TAU_M ) */
  #define __GATE_M_PAR	__T_M_PAR
#else
  #define __GATE_M_PAR	__BASIC_PARS
#endif /* __GATE_M */
#ifdef __GATE_H
  #ifdef __GEN_ALPHA_BETA_H
     #define __ALT_H_PARS	__APPEND_NAME(__GATE_M_PAR, _par_alpha_h_A\t_par_alpha_h_B\t_par_alpha_h_C\t_par_alpha_h_D\t_par_alpha_h_E\t_par_beta_h_A\t_par_beta_h_B\t_par_beta_h_C\t_par_beta_h_D\t_par_beta_h_E)
	_par_alpha_h_A,			//
	_par_alpha_h_B,			//
	_par_alpha_h_C,			//
	_par_alpha_h_D,			//
	_par_alpha_h_E,			//
	_par_beta_h_A,			//
	_par_beta_h_B,			//
	_par_beta_h_C,			//
	_par_beta_h_D,			//
	_par_beta_h_E,			//
  #elif defined( __GENERIC_H )
	_par_gen_h_v12,			// half-voltage
	_par_gen_h_k,			//
     #ifdef __TAU_H
        #define __ALT_H_PARS	__APPEND_NAME(__GATE_M_PAR, _par_gen_h_v12\t_par_gen_h_k\t_par_gen_h_Tmax\t_par_gen_h_Tmax1\t_par_gen_h_Tmax2)
	_par_gen_h_Tmax,		//
	_par_gen_h_Tmax1,		//
	_par_gen_h_Tmax2,		//
     #else
        #define __ALT_H_PARS	__APPEND_NAME(__GATE_M_PAR, _par_gen_h_v12\t_par_gen_h_k)
     #endif /* __TAU_H */
  #else
     #define __ALT_H_PARS	__GATE_M_PAR
  #endif /* __GEN_ALPHA_BETA_H */
  #ifdef __GENERIC_POW_H
     #define __POW_H_PAR	__APPEND_NAME(__ALT_H_PARS,_par_h_pow)
	_par_h_pow,			//
  #else
     #define __POW_H_PAR	__ALT_H_PARS
  #endif /* __GENERIC_POW_H */
  #if defined( __TEMPERATURE_H ) && defined ( __TAU_H )
     #define __T_H_PAR	__APPEND_NAME(__POW_H_PAR,h temperature coefficient)
	_par_h_temperature,		// temperature coefficient
  #else
     #define __T_H_PAR	__POW_H_PAR
  #endif /* defined( __TEMPERATURE_H ) && defined ( __TAU_H ) */
     #define __GATE_H_PAR	__T_H_PAR
#else
  #define __GATE_H_PAR	__GATE_M_PAR
#endif /* __GATE_H */
// setup variables and modvar string
#define __BASIC_VARS	Membrane potential\tCell current\tChannel current\tCell conductance\tg*E
	_var_voltage,			// membrane potential
	_var_total_current,		// cell current
	_var_current,			// channel current
	_var_total_conduct,
	_var_total_GE,
#ifdef __GATE_M
	_var_gate_m,			// gate variable (activation)
  #define __GATE_M_VAR	__APPEND_NAME(__BASIC_VARS,Activation)
#else
  #define __GATE_M_VAR	__BASIC_VARS
#endif /* __GATE_M */
#ifdef __GATE_H
	_var_gate_h,			// gate variable (inactivation)
  #define __GATE_H_VAR	__APPEND_NAME(__GATE_M_VAR,Inactivation)
#else
  #define __GATE_H_VAR	__GATE_M_VAR
#endif /* __GATE_H */
#ifdef __DBFLAG__
   #ifdef __GATE_M
	_var_m_inf,		// infinity gate variable (activation)
	_var_m_tau,
        #define __DB_GATE_M_VAR	__APPEND_NAME(__GATE_H_VAR,Activation steady state\tActivation time constant)
   #else /* __GATE_M */
	#define __DB_GATE_M_VAR	__GATE_H_VAR
   #endif /* __GATE_M */
   #ifdef __GATE_H
	_var_h_inf,		// infinity gate variable (inactivation)
	_var_h_tau,
        #define __DB_GATE_H_VAR	__APPEND_NAME(__DB_GATE_M_VAR,Inactivation steady state\tInactivation time constant)
   #else /* __GATE_H */
	#define __DB_GATE_H_VAR	__DB_GATE_M_VAR
   #endif /* __GATE_H */
   #define __DB_VAR  __DB_GATE_H_VAR
#else /* __DBFLAG__ */
   #define __DB_VAR  __GATE_H_VAR
#endif /* __DBFLAG__ */


#ifdef __ACCOMULATE
	_var_current_accomulate,
	#define __ACCOMULATE_VAR __APPEND_NAME(__DB_VAR,accomulate ion current)
#else
	#define __ACCOMULATE_VAR __DB_VAR
#endif
	__end_par_var__,		// 
};

#define __MOD_PARS	__XSTRING(__ADD_NULL(__GATE_H_PAR))
#define __MOD_VARS	__XSTRING(__ADD_NULL( __ACCOMULATE_VAR))

typedef struct __block{
	__MODULE					//
	dptr	parvar[__end_par_var__];		//
	struct __block *nextblk;			//
} block;

const enum __mt	modtype		= __channel;		//
const enum __ml	modmulti	= __single;		//
const int	moditer		= 1;			//
const int	modprior	= 2;			//
const int	modnames	= __end_par_var__;	//
const int	modblocksize	= sizeof(block);	//
const double	modweight	= -1.0;			//
#ifdef __MOD_NAME
const char	modname[]	= __MOD_NAME;		//
#else
const char	modname[] 	= __FILE__;
#endif /* __MOD_NAME */
#ifdef __MOD_DESCRIPTION
const char      moddescription[]=  __MOD_DESCRIPTION
"\nThe different equasion are solved by exponential Eulear method\0";
#else
const char	moddescription[]= "Automatically created from "__FILE__;
#endif /* __MOD_DESCRIPTION */
const char	modpar[]	= __MOD_PARS;		//
const char	modvar[] 	= __MOD_VARS;		//

static int mid = -1;					// module id
static block *rootblk = NULL;				//
static int iteration = 0;				//
#ifdef __MOD_INIT
static	int init_base( int imid )
#else
int init( int imid )
#endif /* __MOD_INIT */
{
	mid = imid;
	rootblk = NULL;
	iteration = 0;
	return 0;
}
#ifdef __MOD_RELEASE
static	int release_base( int imid  )
#else
int release( int imid )
#endif /* __MOD_RELEASE */
{
	//The module uses the static memory function mem_smalloc()
	//Thus all resources must be freed by memmanager.c;
	rootblk = NULL;
	return 0;
}
#ifdef __MOD_ADD
static	module *add_base( int imid,  module *next, void *mem_handle )
#else
module *add( int imid,  module *next, void *mem_handle )
#endif /* __MOD_ADD */
{
	block * newblk = ( block *)mem_g_malloc(mem_handle, sizeof( block ));
	if( newblk == NULL )
		return ( module *) NULL; /*Cannot allocate memory*/
	newblk->nextblk = rootblk;
	rootblk = newblk;
	int cnt; for( cnt = 0; cnt < __end_par_var__; rootblk->parvar[cnt++].dp = NULL );
	rootblk->mid = mid;
	rootblk->next = next;
	return ( module *)rootblk;
}
#ifdef __MOD_CALCULATE
static int calculate_base( int imid )
#else
int calculate( int imid )
#endif /* __MOD_CALCULATE */
{
	if(rootblk == NULL) return -1;
	block *currentblk = rootblk;
	datum voltage;
	datum  newconduct, newcur;
#ifdef __GATE_M
	datum newgatem, alpham, tm, m;
#endif
#ifdef __GATE_H
	datum newgateh, alphah, th, h;
#endif
	for( ; currentblk != NULL; currentblk = currentblk->nextblk ){
		voltage = GN_SW( _var_voltage );
		newconduct = GN_SW( _par_mconduct );
#ifdef __GATE_M
  #if defined( __ALPHA_M ) && defined ( __BETA_M )
		alpham = __ALPHA_M( voltage );
    #ifdef  __TAU_M 
       #ifdef __CONST_TAU_M
		m = alpham/( alpham+__BETA_M( voltage ) );
		newgatem = ( GN_SW( _var_gate_m )-m )*exp( -time_step /(__CONST_TAU_M) )+m;
            #ifdef __DBFLAG__
		GNASW(_var_m_inf) = m;
		GNASW(_var_m_tau) = __CONST_TAU_M;
            #endif /*__DBFLAG__*/
	#else
		tm = 1./( alpham+__BETA_M( voltage ) );
		m = alpham*tm;
            #ifdef __TEMPERATURE_M
		tm *= GN_SW( _par_m_temperature );
            #endif /* __TEMPERATURE_M */
		newgatem = ( GN_SW( _var_gate_m )-m )*exp( -time_step /tm )+m;
            #ifdef __DBFLAG__
		GNASW(_var_m_inf) = m;
		GNASW(_var_m_tau) = tm;
            #endif /*__DBFLAG__*/
	#endif
     #else
		newgatem = alpham/( alpham+__BETA_M( voltage ));
            #ifdef __DBFLAG__
		GNASW(_var_m_inf) = newgatem;
		GNASW(_var_m_tau) = 0.0;
            #endif /*__DBFLAG__*/
     #endif /* __TAU_M */
  #elif defined( __INF_M )
      #ifdef __TAU_M
	 #ifdef __CONST_TAU_M
		m = __INF_M( voltage );
		newgatem = ( GN_SW( _var_gate_m )-m )*exp( -time_step /(__CONST_TAU_M) )+m;
            #ifdef __DBFLAG__
		GNASW(_var_m_inf) = m;
		GNASW(_var_m_tau) = __CONST_TAU_M;
            #endif /*__DBFLAG__*/
         #else
 		m = __INF_M( voltage );
		tm = __TAU_M( voltage );
	   #ifdef __TEMPERATURE_M
		tm *= GN_SW( _par_m_temperature );
           #endif /* __TEMPERATURE_M */
		newgatem = ( GN_SW( _var_gate_m )-m )*exp( -time_step /tm )+m;
            #ifdef __DBFLAG__
		GNASW(_var_m_inf) = m;
		GNASW(_var_m_tau) = tm;
            #endif /*__DBFLAG__*/
	 #endif
     #else
		newgatem = __INF_M( voltage );
            #ifdef __DBFLAG__
		GNASW(_var_m_inf) = newgatem;
		GNASW(_var_m_tau) = 0.0;
            #endif /*__DBFLAG__*/
     #endif /* __TAU_M */
  #else
		newgatem = 1.;
            #ifdef __DBFLAG__
		GNASW(_var_m_inf) = 1.0;
		GNASW(_var_m_tau) = 0.0;
            #endif /*__DBFLAG__*/
  #endif /* defined( __ALPHA_M ) && defined ( __BETA_M ) */
		GNASW( _var_gate_m ) = newgatem;
		newconduct *= __POW_M( newgatem );
#endif /*__GATE_M */
#ifdef __GATE_H
  #if defined( __ALPHA_H ) && defined ( __BETA_H )
		alphah = __ALPHA_H( voltage );
     #ifdef __TAU_H
	#ifdef __CONST_TAU_H
		h = alphah/( alphah+__BETA_H( voltage ) );
		newgateh = ( GN_SW( _var_gate_h )-h )*exp( -time_step /(__CONST_TAU_H) )+h;
            #ifdef __DBFLAG__
		GNASW(_var_h_inf) = h;
		GNASW(_var_h_tau) = __CONST_TAU_H;
            #endif /*__DBFLAG__*/
	#else
		th = 1./( alphah+__BETA_H( voltage ) );
		h = alphah*th;
          #ifdef __TEMPERATURE_H
		th *= GN_SW( _par_h_temperature );
          #endif /* __TEMPERATURE_H */
		newgateh = ( GN_SW( _var_gate_h )-h )*exp( -time_step /th )+h;
            #ifdef __DBFLAG__
		GNASW(_var_h_inf) = h;
		GNASW(_var_h_tau) = th;
            #endif /*__DBFLAG__*/
	#endif
     #else
		newgateh = alphah/( alphah+__BETA_H( voltage ) );
            #ifdef __DBFLAG__
		GNASW(_var_h_inf) = newgateh;
		GNASW(_var_h_tau) = 0.0;
            #endif /*__DBFLAG__*/
     #endif /* __TAU_H */
  #elif defined( __INF_H )
     #ifdef __TAU_H
	#ifdef __CONST_TAU_H
		h = __INF_H( voltage );
		newgateh = ( GN_SW( _var_gate_h )-h )*exp( -time_step /(__CONST_TAU_H) )+h;
            #ifdef __DBFLAG__
		GNASW(_var_h_inf) = h;
		GNASW(_var_h_tau) = __CONST_TAU_H;
            #endif /*__DBFLAG__*/
	#else
		h = __INF_H( voltage );
		th = __TAU_H( voltage );
          #ifdef __TEMPERATURE_H
		th *= GN_SW( _par_h_temperature );
          #endif /* __TEMPERATURE_H */
		newgateh = ( GN_SW( _var_gate_h )-h )*exp( -time_step /th )+h;
            #ifdef __DBFLAG__
		GNASW(_var_h_inf) = h;
		GNASW(_var_h_tau) = th;
            #endif /*__DBFLAG__*/
	#endif
     #else
		newgateh = __INF_H( voltage );
            #ifdef __DBFLAG__
		GNASW(_var_h_inf) = newgateh;
		GNASW(_var_h_tau) = 0.0;
            #endif /*__DBFLAG__*/
     #endif /* __TAU_M */
  #else
		newgateh = 1.;
            #ifdef __DBFLAG__
		GNASW(_var_h_inf) = 1.0;
		GNASW(_var_h_tau) = 0.0;
            #endif /*__DBFLAG__*/
  #endif /* defined( __ALPHA_H ) && defined ( __BETA_H ) */
		GNASW( _var_gate_h ) = newgateh;
		newconduct *= __POW_H( newgateh );
#endif /* __GATE_H */
		GNASW( _var_total_conduct ) += newconduct;
		GNASW( _var_total_GE ) += newconduct*GN_SW( _par_rev_pot );
		GNASW( _var_current ) = newcur = newconduct * ( voltage-GN_SW( _par_rev_pot ) );
		GNASW( _var_total_current ) += newcur;
#ifdef __ACCOMULATE
		GNASW( _var_current_accomulate ) += newcur;
#endif /* __ACCOMULATE */
	}
	return iteration;
}
#ifdef __MOD_SLACK
static int slack_base( int imid )
#else
int slack( int imid )
#endif /* __MOD_SLACK */
{
	if( rootblk != NULL )
		return iteration;
	return -1;
}

#ifdef __MOD_RESTORE
static int restore_base( int iMID, int hFile, int oid, int bid )
#else
int restore( int iMID, int hFile, int oid, int bid )
#endif /* __MOD_RESTORE */
{
	return 0;
}

#ifdef __MOD_SAVE
static int save_base( int iMID, int hFile, int oid, int bid )
#else
int save( int iMID, int hFile, int oid, int bid )
#endif /* __MOD_SAVE */
{
/*
	if(rootblk == NULL) return -1;
	block *currentblk = rootblk;
	double voltage;
	double  newconduct, newcur;
	for( ; currentblk != NULL; currentblk = currentblk->nextblk ){
#ifdef __GATE_M
	double newgatem, alpham, tm, m;
#endif
#ifdef __GATE_H
	double newgateh, alphah, th, h;
#endif
	}
*/
	return 0;
}
