/* 
 * objmanager.c version 1.0 beta
 * 
 * Copyright (C) 2005 Ruben Tikidji-Hamburian <rth@nisms.krinc.ru>
 * Copyright (C) 2005 Sergey Markin <markin@nisms.krink.ru>
 * ----------------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ----------------------------------------------------------------------------
 */

#include <string.h>

#include <neurocad/engine/nce_common.h>
#include <neurocad/engine/simmanager.h>
#include <neurocad/assemblers/basic/objmanager.h>

#ifndef OBJECT_HASH_STEP
#	define OBJECT_HASH_STEP  5
#endif

typedef struct pro_objcont{
	long uptoID;			// maximal object id for hash branch
	obp obj;			// a pointer to object in the linked list
	struct pro_objcont *down_level;	// a pointer to next item at the down layer
	struct pro_objcont *next;		// a pointer to next item at the same layer
} objcont;

typedef objcont *ocp;

///////////////////////////////////////////////////////////////////////////////
//	local variables
static ocp hd_head = NULL;		// a head of a hash table
static ocp hd_root = NULL;		// a basic pointer to a hash table

///////////////////////////////////////////////////////////////////////////////
//	global variables
long int  obj_number = 0;		// total amount of objects in a linked list
obp obj_root = NULL;			// root of a linked list
obp obj_curr = NULL;			// current node in a linked list
void *memory_heandle = NULL;		//Memory heandler

///////////////////////////////////////////////////////////////////////////////
//	local functions
//-----------------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////////////
//	find_ptr() tries to find out the object through the linked list.
//	Returns a pointer to the object if it is exist
//	overwise returns NULL
//	Parameters: 	oid	 is id of the object to be finded 
static obp find_ptr( unsigned long oid )
{
	obp retptr = NULL;
	// a search through the hash table
	ocp hdptr = hd_head;
	while( hdptr ){
		if( hdptr->uptoID >= oid ){
			if( hdptr->down_level )	
				hdptr = hdptr->down_level;
			else{
				retptr = hdptr->obj;
				hdptr = NULL;
			}
		}
		else
			hdptr = hdptr->next;
	}
	// a search through the linked list
	if( retptr == NULL ){
		if( obj_curr == NULL )
			obj_curr = obj_root;
		if( obj_curr != NULL )
			retptr = ( obj_curr->oid < oid ) ? obj_curr : obj_root;
	}
	for( ; ( retptr != NULL )&&( retptr->oid < oid ); retptr = retptr->next );
	if(( retptr == NULL ) || ( retptr->oid != oid ))
		return ( obp )NULL;
	return retptr;
}

///////////////////////////////////////////////////////////////////////////////
//	free_list() removes all object from the linked list
//	Always returns 0.
static int free_list( obp root  )
{
	if( root ){
		if( root->data )
			mem_free( root->data );
		obp curptr = root;
		for( ; curptr != NULL; ){
			obp tmp = curptr->next;
	 		mem_free( curptr );
			curptr = tmp;
		}
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//	alloc_hd() allocates static memory for a linked list, named variables,
//	and a hash table.
//	Returns 0 if success 
//	overwise returns a negative value.
//	Parameters: 	all_names	is total amount of named variables for all 
//					objects in the linked list.
//			hash_size	is size of a hash table
static int alloc_hd( unsigned long all_names, unsigned long all_block_size )
{
	// calculate a size of a hash table
	unsigned long hashsize = 0;
	unsigned long counter = obj_number;
	
	do{
		if( counter%OBJECT_HASH_STEP != 0 )
			counter += OBJECT_HASH_STEP;
		counter /= OBJECT_HASH_STEP;
		hashsize += counter;
	} while( counter > OBJECT_HASH_STEP );
	
	
	// calculate max size of nce_bufdeep
	size_t freemem = engine_context.memsize_free-( all_block_size+
					obj_number*sizeof( object )+
					hashsize*sizeof( objcont ));
	unsigned long maxsize = freemem/( all_names*sizeof( datum ));
	if( maxsize < nce_bufdeep ){
		nce_bufdeep = maxsize;
		WARNING_MESSAGE( "alloc_hd", "\'nce_bufdeep\' has been changed", NULL, 0 );
	}
	if( nce_bufdeep < 3 )
		return ERROR_MESSAGE( "alloc_hd", "There is not enough memory for data", NULL, -1 );
	// calculate the total amount of memory to be allocated for a model data
	size_t total_size = 	all_block_size+				// blocks
				obj_number*sizeof( object )+		// objects
				hashsize*sizeof( objcont )+		// hash table
				all_names*nce_bufdeep*sizeof( datum );	// named variables
	// allocate static memory for model data
	if( ( memory_heandle = mem_g_init( total_size )) == NULL )
		return ERROR_MESSAGE( "alloc_hd", "Cannot allocate static memory for model data. \'mem_g_init()\' returns an error.", NULL, -2 );
	
	// allocate static memory for the linked list
	obp newroot = NULL;
	if(( newroot = ( obp )mem_g_malloc(memory_heandle, obj_number*sizeof( object ))) == NULL )
		return ERROR_MESSAGE( "alloc_hd", "Cannot allocate memory for a linked list of objects. \'mem_s_malloc()\' returns an error.", NULL, -3 );
	// copy the linked list to static memory
	obp objptr = obj_root,
	    newobj = newroot;
	for( ; objptr != NULL; objptr = objptr->next ){
		*newobj = *objptr;
		if( objptr->next ){
			newobj->next = newobj+1;
			newobj++;
		}
	}
	newobj->next = NULL;
	free_list( obj_root );
	obj_root = obj_curr = newroot;
	// allocate static memory for named variables
	if(( obj_root->data = ( datum *)mem_g_malloc(memory_heandle, all_names*nce_bufdeep*sizeof( datum ))) == NULL )
		return ERROR_MESSAGE( "alloc_hd","Cannot allocate memory for all named variables. \'mem_s_malloc()\' returns an error.", NULL, -4 );
	// allocate static memory for a hash table
	if(( hd_root = ( ocp )mem_g_malloc(memory_heandle, hashsize*sizeof( objcont ))) == NULL )
		return ERROR_MESSAGE( "alloc_hd","Cannot allocate memory for a hash table. \'mem_s_malloc()\' returns an error.", NULL, -5 );
	memset( hd_root, 0, hashsize*sizeof( objcont ));
	hd_head = hd_root+hashsize-1;		// point to last cell in a hash table
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//	get_hd_branch() gets information about a branch of the hash table which
//		correspondes to an object in the linked list
//	Returns 0 if success 
//		overwise returns -1
//	Parameters:	obj	 is pointer to an object in the linked list.
//			hd_layer is pointer to a branch of the hash table which 
//			         correspondes to obj. It must be filled out
//			hd_deep  is maximal order of the hash table
static int get_hd_branch( obp obj, ocp *hd_layer, unsigned long hd_deep )
{
	if( obj == NULL )
		return -1;
	memset( hd_layer, 0, sizeof( ocp )*( hd_deep+1 ));
	ocp hdptr = hd_head;
	while( hdptr && hd_deep >=0 ){
		if( hdptr->uptoID >= obj->oid ){
			if( hdptr->down_level ){
				hd_layer[--hd_deep] = hdptr;
				hdptr = hdptr->down_level;
			}
			else if( hdptr->obj == obj ){
				hd_layer[0] = hdptr;
				return 0;
			}
			else{
				return -1;
			}
		}
		else{
			hdptr = hdptr->next;
			hd_layer[hd_deep] = NULL;
		}
	}
	return -1;
}

///////////////////////////////////////////////////////////////////////////////
//	global functions
//-----------------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////////////
//	obj_add() adds new object to the linked list
//	If success it returns total amount of object in the linked list
//	overwise returns a negative value
//	Parameters: 	oid	is id of the object to be added to the linked 
//				list.
long obj_add( long oid )
{
	if( hd_root != NULL )
		return ERROR_MESSAGE( "obj_add", "The linked list of objects was initialized before.", NULL, -1 );
	char FnArg[21]; memset( FnArg, 0, 21 );
	snprintf( FnArg, 20, "%ld", oid );
	if( find_ptr( oid ))
		return ERROR_MESSAGE( "obj_add","The object with such \"id\" has been added to the list before.", FnArg, -2 );

	obp objnew = ( obp )mem_malloc( sizeof( object ));
	if( objnew == NULL )
		return ERROR_MESSAGE( "obj_add", "Cannot allocate memory for new object. \'mem_malloc()\' returns an error.", FnArg, -3 );
	memset( objnew, 0, sizeof( object ));
	objnew->oid = oid;
	if( obj_root == NULL ){
		obj_root = obj_curr = objnew;
	}
	else if( obj_root->oid > oid ){
		objnew->next = obj_root;
		obj_root = obj_curr = objnew;
	}
	else{
		if( obj_curr == NULL )
			obj_curr = obj_root;
		obj_curr = ( obj_curr->oid < oid ) ? obj_curr : obj_root;
		for(; ( obj_curr->next != NULL )&&( obj_curr->next->oid < oid ); obj_curr = obj_curr->next);
		if( obj_curr->next == NULL ){
			obj_curr->next = objnew;
			obj_curr = obj_curr->next;
		}
		else{
			objnew->next = obj_curr->next;
			obj_curr->next = objnew;
		}
	}
	return ++obj_number;
}

///////////////////////////////////////////////////////////////////////////////
//	obj_inithd() allocates memory for all named variables in the linked 
//	list and builds up a hash table.
//	Returns 0 if success 
//	overwise it returns a negative value.
//	Parameters: 	all_names	 is total amount of named variables for 
//					 all objects in the linked list.
//			all_block_size	is total size of all blocks.
int obj_inithd( unsigned long all_names, size_t all_block_size )
{
	if( hd_root != NULL )
		return ERROR_MESSAGE( "obj_inithd","The linked list of objects was initialized before.", NULL, -1 );
	if( alloc_hd( all_names, all_block_size ) < 0 )
		return ERROR_MESSAGE( "obj_inithd","\'alloc_hd()\' returns an error.", NULL, -2 );
	// go through the linked list and set up the data pointers and the first layer of the hash table
	unsigned long counter = 0;
	size_t offset = 0;
	ocp hdptr = hd_head;
	obp objptr = obj_curr = obj_root;
	for( counter = 0, hdptr->obj = objptr, hdptr->uptoID = objptr->oid; objptr != NULL; objptr = objptr->next, ++counter ){
		objptr->data = obj_root->data+offset;
		offset += objptr->datasize*nce_bufdeep;
		if( offset > ( size_t )( all_names*nce_bufdeep ))
			return ERROR_MESSAGE( "obj_inithd","It is not enough allocated memory for all data.", NULL, -3 );
		if( hdptr != hd_root ){
			if( counter%OBJECT_HASH_STEP == 0 ){
				if( counter ){
					hdptr->next = hdptr-1;	// pick up next empty cell
					--hdptr;
				}
				hdptr->obj = objptr;
			}
			hdptr->uptoID = objptr->oid;
		}
	}
	// set up the last layers of the hash table
	while(( hdptr != hd_root )&&( counter > OBJECT_HASH_STEP )){
		ocp hdown = hd_head;	// save a previous head of the hash table
          	hd_head = --hdptr;	// a head of the hash table points to the first empty cell now
		for( counter = 0, hdptr->down_level = hdown, hdptr->uptoID = hdown->uptoID; ( hdown != NULL )&&( hdptr >= hd_root ); hdown = hdown->next, ++counter ){
			if( counter%OBJECT_HASH_STEP == 0 ){
				if( counter ){
					hdptr->next = hdptr-1;	// pick up next empty cell
					--hdptr;
					}
				hdptr->down_level = hdown;
			}
			hdptr->uptoID = hdown->uptoID;
		}
	}
	return 0;	
}

///////////////////////////////////////////////////////////////////////////////
//	obj_freeall() erase all data allocated by alloc_hd(). Memory MUST BE 
//	FREED by mem_g_releasel().
//	Always returns 0.
int obj_releasehd( void )
{
	obj_number = 0;
	obj_root = obj_curr = NULL;
	hd_root = hd_head = NULL;
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//	obj_find() tries to find out the object through the linked list.
//	Returns a pointer to the object if it is exist
//	overwise returns NULL.
//	Parameters: 	oid	 is id of the object to be finded.
obp obj_find(  long oid )
{
	char FnArg[21]; memset( FnArg, 0, 21 );
	snprintf( FnArg, 20, "%ld", oid);
	if( obj_root == NULL )
		return ( obp )ERROR_MESSAGE("obj_find","The objects list is empty.", FnArg, 0 );
	return find_ptr( oid );
}

///////////////////////////////////////////////////////////////////////////////
//	obj_setactive() tries to set up the object (oid) as an active one.
//	Returns id of the object if success 
//	overwise returns -1.
//	Parameters:	oid	 is id of the object to be set up.  
long obj_setactive( long oid )
{
	obp retptr = obj_find( oid );
	if( retptr ){
		obj_curr = retptr;
		return obj_curr->oid;
	}
	char FnArg[21]; memset( FnArg,0,21 );
	snprintf( FnArg, 20, "%ld", oid );
	return WARNING_MESSAGE( "obj_set_active", "Cannot find the object with required id.", FnArg, -1 );
}

///////////////////////////////////////////////////////////////////////////////
//	obj_test() tests if the object (oid) presents in the linked list.
//	If success it returns 0 
//	overwise it returns a negative value.
//	Parameters:	oid	 is id of the object to be tested.
int obj_test( long oid )
{
	char FnArg[21]; memset( FnArg, 0, 21 );
	snprintf( FnArg, 20, "%ld", oid );
	if( obj_root == NULL )
		return ERROR_MESSAGE( "obj_test","The linked list is empty.", FnArg, -1 );
	if( obj_setactive( oid ) < 0 )
		return ERROR_MESSAGE( "obj_test", "\'obj_set_active()\' returns an error.", FnArg, -2 );
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//	obj_testall() tests all objects in the linked list.
//	If success it returns 0 
//	overwise it returns a negative value.
int obj_testall( void )
{
	int exit_code = 0;
	obp objptr = obj_root;
	for( ; objptr != NULL && exit_code == 0; objptr = objptr->next )
		exit_code = obj_test( objptr->oid );
	return exit_code;
}

///////////////////////////////////////////////////////////////////////////////
//	obj_getdata() 
datum obj_getdata( long oid, size_t nid )
{
	char FnArg[21]; memset( FnArg, 0, 21 );
	snprintf( FnArg, 20, "%ld, %lu", oid, ( unsigned long )nid );
	if ( obj_setactive( oid ) != oid )
		return (datum) ERROR_MESSAGE( "obj_getdata", "\'obj_set_active()\' returns an error.", FnArg, (int) 0.0 );
	if( nid >= obj_curr->datasize )
		return (datum) ERROR_MESSAGE( "obj_getdata", "The address is out of the object datasize.", FnArg, (int) 0.0 );
	return obj_curr->data[nce_bufdeep*nid+nce_sw];
}

///////////////////////////////////////////////////////////////////////////////
//	obj_getbuffer()
datum obj_getbuffer( long oid, size_t nid, int shift )
{
	char FnArg[21]; memset( FnArg, 0, 21 );
	snprintf( FnArg, 20, "%ld, %lu", oid, ( unsigned long )nid );
	if ( obj_setactive( oid ) != oid )
		return (datum) ERROR_MESSAGE( "obj_getdata", "\'obj_set_active()\' returns an error.", FnArg, (int) 0.0 );
	if( nid >= obj_curr->datasize )
		return (datum) ERROR_MESSAGE( "obj_getdata", "The address is out of the object datasize.", FnArg, (int) 0.0 );
	return obj_curr->data[ nce_bufdeep*nid + ( (nce_sw + shift) % nce_bufdeep ) ];
}

///////////////////////////////////////////////////////////////////////////////
//	obj_setdata() 
int obj_setdata( long oid, size_t nid, datum value )
{
	char FnArg[41]; memset( FnArg, 0, 41);
	snprintf( FnArg, 40, "%ld, %lu, %lf", oid, ( unsigned long )nid, value );
	if ( obj_setactive(oid) != oid )
		return ERROR_MESSAGE( "obj_setdata", "\'obj_set_active()\' returns an error.", FnArg, -1 );
	if( nid >= obj_curr->datasize )
		return ERROR_MESSAGE( "obj_setdata", "The address is out of the object datasize.", FnArg, -2 );
	int buff_cnt;
	for( buff_cnt = 0; buff_cnt < nce_bufdeep; buff_cnt++){
		obj_curr->data[nce_bufdeep*nid+buff_cnt] = value;
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//	obj_getdatapointer()
datum	*obj_getdatapointer( long oid, size_t nid)
{
	char FnArg[21]; memset( FnArg, 0, 21 );
	snprintf( FnArg, 20, "%ld, %lu", oid, ( unsigned long )nid );
	if ( obj_setactive( oid ) != oid )
		return (datum *) ERROR_MESSAGE( "obj_getdatapointer", "\'obj_set_active()\' returns an error.", FnArg, (int)NULL );
	if( nid >= obj_curr->datasize )
		return (datum *) ERROR_MESSAGE( "obj_getdatapointer", "The address is out of the object datasize.", FnArg, (int)NULL );
	return obj_curr->data + nce_bufdeep * nid;
}

///////////////////////////////////////////////////////////////////////////////
//	obj_print() prints out information about the linked list and the hash 
//	table.
//	Always returns 0.
int obj_printinfo( void )
{
	// calculate a maximal order of the hash table
	unsigned long counter = 0;
	unsigned long hd_deep = 1;
	ocp hdptr = hd_head;
	if( hdptr != NULL )
		for( ; hdptr->down_level != NULL; hdptr = hdptr->down_level, ++hd_deep );
	// print out the general information of the linked list
	OUTPUT_MESSAGE("\nObject manager statistics");
	OUTPUT_MESSAGE("\n-------------------------");
	OUTPUT_MESSAGE("\nTotal amount of objects in the linked list is %ld\nThe root of the linked list is %lX\nThe current node in the linked list is %lX\n", obj_number, ( unsigned long )obj_root, ( unsigned long )obj_curr );
	OUTPUT_MESSAGE("\nObjects table");
	// print out the table header 
	OUTPUT_MESSAGE("\n===========================================");
	for( counter = 0; counter < hd_deep; ++counter )
		OUTPUT_MESSAGE("===============================================");
	OUTPUT_MESSAGE("\n|_____________   OBJECT   _________________");
	for( counter = 0; counter < hd_deep; ++counter )
		OUTPUT_MESSAGE("|______________ HASH (level %ld) ________________", counter );
	OUTPUT_MESSAGE("\n[    id: address:    data:  size:    next] ");
	for( counter = 0; counter < hd_deep; ++counter )
		OUTPUT_MESSAGE("| [uptoID: address:  object: d_level:    next] ");
		
	OUTPUT_MESSAGE("\n-------------------------------------------");
	for( counter = 0; counter < hd_deep; ++counter )
		OUTPUT_MESSAGE("-----------------------------------------------");
	OUTPUT_MESSAGE("\n\n");
	// print out the linked list and the hash table
	obp objptr = obj_root;
	ocp hd_layer[hd_deep+1];
	int is_hash = get_hd_branch( objptr, hd_layer, hd_deep );
	for( ; objptr != NULL; ){
		OUTPUT_MESSAGE( "[%06ld:%08lX:%08lX:%06ld:%08lX]",  objptr->oid,( unsigned long )objptr, ( unsigned long )objptr->data, ( unsigned long )objptr->datasize, ( unsigned long )objptr->next );
		if( is_hash == 0 ){
			OUTPUT_MESSAGE( "<--" );
			for( counter = 0; counter < hd_deep && hd_layer[counter]; ++counter ){
				OUTPUT_MESSAGE( "[%06ld:%08lX:%08lX:%08lX:%08lX]",  hd_layer[counter]->uptoID,( unsigned long )hd_layer[counter], ( unsigned long )hd_layer[counter]->obj, ( unsigned long )hd_layer[counter]->down_level, ( unsigned long )hd_layer[counter]->next );
				if( hd_layer[counter+1] )
					OUTPUT_MESSAGE( "<--" );
				else if( counter < hd_deep-1 ){
					OUTPUT_MESSAGE( "                      |" );
					int counter1 = counter+1;
					for( ; counter1 < hd_deep-1; ++counter1 )
						OUTPUT_MESSAGE( "  %45s", "|" );
				}
			}
		}
		else{
			OUTPUT_MESSAGE("                      |");
			for( counter = 1; counter < hd_deep; ++counter )
				OUTPUT_MESSAGE( "  %45s", "|" );
		}
		OUTPUT_MESSAGE( "\n%20s", "|" );
		for( counter = 0; counter < hd_deep; ++counter )
				OUTPUT_MESSAGE( "%45s  ", "|" );
		objptr = objptr->next;
		is_hash = get_hd_branch( objptr, hd_layer, hd_deep );
		if( objptr ){
			OUTPUT_MESSAGE( "\n%20s", "v" );
			for( counter = 0; counter < hd_deep && hd_layer[counter]; ++counter )
				OUTPUT_MESSAGE( "%45s  ", "v" );
			for( ; counter < hd_deep; ++counter )
				OUTPUT_MESSAGE( "%45s  ", "|" );
		}
		OUTPUT_MESSAGE("\n");
	}
	OUTPUT_MESSAGE( "%20s", "X" );
	for( counter = 0; counter < hd_deep; ++counter )
		OUTPUT_MESSAGE( "%45s  ", "X" );
	OUTPUT_MESSAGE("\n-------------------------------------------");
	for( counter = 0; counter < hd_deep; ++counter )
		OUTPUT_MESSAGE("-----------------------------------------------");
	OUTPUT_MESSAGE("\n");
	return 0;
}
